import os
import sys

import gi
gi.require_version("Gst", "1.0")
from gi.repository import Gst


# CONSTANTS
# =========

DEFAULT_VIS_PLUGIN = "Waveform oscilloscope"


# FUNCTIONS
# =========

def get_vis_plugin(name):
    """Return the visualization plugin called NAME or None."""
    registry = Gst.Registry.get()
    vis_plugins = registry.feature_filter(is_visualizer, False)
    vis_plugin = None

    for plugin in vis_plugins:
        short_name = plugin.get_plugin_name()
        long_name = plugin.get_metadata("long-name")
        if short_name == name or long_name == name:
            vis_plugin = plugin.create()

    return vis_plugin


def is_visualizer(feature):
    """Return True if FEATURE's klass is Visualization."""
    return (
        isinstance(feature, Gst.ElementFactory) and
        "Visualization" in feature.get_metadata("klass")
    )


# MAIN
# ====

# Initialize GStreamer.
Gst.init(None)

# Define audio file location.
audio_file_path = os.path.abspath("song18.mp3")

# Build a playbin pipeline.
textual_pipeline = "playbin uri=file://{}".format(audio_file_path)
pipeline = Gst.parse_launch(textual_pipeline)

# Set the visualization flag.
flags = pipeline.get_property("flags")
vis_flag = (1 << 3)
flags |= vis_flag
pipeline.set_property("flags", flags)

# Set visualization plugin.
vis_plugin = get_vis_plugin(DEFAULT_VIS_PLUGIN)

if len(sys.argv) == 2:
    # A plugin name was passed as argument on CLI, use it.
    vis_plugin = get_vis_plugin(sys.argv[1])

pipeline.set_property("vis-plugin", vis_plugin)

# Start playing.
pipeline.set_state(Gst.State.PLAYING)

# Wait until error or EOS.
bus = pipeline.get_bus()
message = bus.timed_pop_filtered(
   Gst.CLOCK_TIME_NONE,
   Gst.MessageType.ERROR | Gst.MessageType.EOS
)

# Free resources.
pipeline.set_state(Gst.State.NULL)
